unit MainUnit;

{$mode objfpc}{$H+}

{*******************************************************************************
*              Program Form1  for LibSndFile.dll (Windows)                     *
*                                                                              *
*                                                                              *
*                      created 2011-01-24  (C) breakoutbox                     *
*                      latest changes 2021-09-23  (C) breakoutbox              *
*                                                                              *
*            released under  GNU Lesser General Public License (LGPL)          *
*******************************************************************************}

interface

uses
  Interfaces,
  LazUTF8, LConvEncoding,
  Classes, Forms, Controls, Graphics, Dialogs, StdCtrls, SysUtils, ctypes,
  libsndfile, libsndfilestrings;

const VersionStr = ' 0.07';   // now compiled with Lazarus 2.0.12 i386-Win32


type
  { TForm1 }
  TForm1 = class(TForm)
    ButtonGetStrings: TButton;
    ButtonCloseFile: TButton;
    ButtonOpenFile: TButton;
    ButtonTestRead: TButton;
    ButtonGetBroadcastChk: TButton;
    ButtonMaxAllChannels: TButton;
    ButtonMaxNormalized: TButton;
    ButtonMaxSample: TButton;
    ButtonLogStr: TButton;
    ButtonLibVersion: TButton;
    ButtonClose: TButton;
    Edit1: TEdit;
    Label1: TLabel;
    Memo1: TMemo;
    OpenDialog1: TOpenDialog;
    procedure ButtonCloseFileClick(Sender: TObject);
    procedure ButtonGetStringsClick(Sender: TObject);
    procedure ButtonOpenFileClick(Sender: TObject);
    procedure ButtonTestReadClick(Sender: TObject);
    procedure ButtonGetBroadcastChkClick(Sender: TObject);
    procedure ButtonLibVersionClick(Sender: TObject);
    procedure ButtonCloseClick(Sender: TObject);
    procedure ButtonLogStrClick(Sender: TObject);
    procedure ButtonMaxAllChannelsClick(Sender: TObject);
    procedure ButtonMaxNormalizedClick(Sender: TObject);
    procedure ButtonMaxSampleClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var {%H-}CloseAction: TCloseAction);
    procedure FormCreate(Sender: TObject);
  private
    { private declarations }
    SFhandle : TSNDFILE_HANDLE;
    SFinfo   : TSF_INFO;
    procedure Show_SFinfo;
    procedure ActivateButtons( b: boolean);
    procedure WriteStringToMemo( s: String);
  public
    { public declarations }
  end; 

var
  Form1: TForm1;


implementation

{$R *.lfm}


{ --- TForm1 ----------------------------------------------------------------- }
procedure TForm1.FormCreate(Sender: TObject);
begin
  Caption:= Caption +'  V' +VersionStr;

  // we manually load the DLL - on runtime :
  LoadSndFileDll;

  ActivateButtons( FALSE);
end;


procedure TForm1.FormClose(Sender: TObject; var CloseAction: TCloseAction);
begin
  UnLoadSndFileDll;
end;


{ --- Buttons ---------------------------------------------------------------- }
procedure TForm1.ButtonCloseClick(Sender: TObject);
begin
  //only "press" CloseFile if File is open :
  //if ButtonCloseFile.Enabled
  if SFhandle <> NIL
    then ButtonCloseFileClick( Sender);
  close;
end;


procedure TForm1.ButtonOpenFileClick(Sender: TObject);
begin
  if OpenDialog1.Execute
  then
    begin
      Memo1.Clear;
      Memo1.Append( 'Informations about "' +OpenDialog1.FileName +'"');
      Memo1.Append( '');
      Edit1.Caption:= OpenDialog1.FileName;

      //automatically close a previously opened file here :
      if SFhandle <> NIL then sf_close( SFhandle);

      SFhandle:= sf_open( UTF8ToCP1252( OpenDialog1.FileName), SFM_READ, SFinfo);
      If SFhandle = NIL then
        begin
          Memo1.Append( 'ERROR: SoundFile Handle = Nil');
          Memo1.Append( sf_strerror( SFhandle));
          Memo1.Append( '');
          Exit;
        end;

      Self.Show_SFinfo;
      Self.ActivateButtons( TRUE);
    end;
end;


procedure TForm1.ButtonCloseFileClick(Sender: TObject);
begin
  sf_close( SFhandle);

  //set pointer no NIL to prevent invalid access :
  SFhandle:= NIL;
  //ShowMessage( IntToStr( dword( SFhandle)));

  Self.ActivateButtons( FALSE);
end;


procedure TForm1.ButtonGetStringsClick(Sender: TObject);
var
  s : string;//pChar;
begin
  //** String types that can be set and read from files. Not all file types
  //** support this and even the file types which support one, may not support
  //** all string types. AIFF and WAV do.
  Memo1.Append( '------------------------------------');
  Memo1.Append( '..... sf_get_string result: .....');
  s:= sf_get_string( SFhandle, SF_STR_TITLE);
  if s = ''
    then Memo1.Append( '<SF_STR_TITLE> ' +#9#9 +'s = ''''  ' +s)
    else Memo1.Append( '<SF_STR_TITLE> ' +#9#9 +s);

  s:= sf_get_string( SFhandle, SF_STR_ARTIST);
  if s = ''
    then Memo1.Append( '<SF_STR_ARTIST> ' +#9 +'s = ''''  ' +s)
    else Memo1.Append( '<SF_STR_ARTIST> ' +#9 +s);

  s:= sf_get_string( SFhandle, SF_STR_COMMENT);
  if s = ''
    then Memo1.Append( '<SF_STR_COMMENT> ' +#9 +'s = ''''  ' +s)
    else Memo1.Append( '<SF_STR_COMMENT> ' +#9 +s);

  s:= sf_get_string( SFhandle, SF_STR_DATE);
  if s = ''
    then Memo1.Append( '<SF_STR_DATE> ' +#9#9 +'s = ''''  ' +s)
    else Memo1.Append( '<SF_STR_DATE> ' +#9#9 +s);

  s:= sf_get_string( SFhandle, SF_STR_COPYRIGHT);
  if s = ''
    then Memo1.Append( '<SF_STR_COPYRIGHT> ' +#9 +'s = ''''  ' +s)
    else Memo1.Append( '<SF_STR_COPYRIGHT> ' +#9 +s);

  s:= sf_get_string( SFhandle, SF_STR_SOFTWARE);
  if s = ''
    then Memo1.Append( '<SF_STR_SOFTWARE> ' +#9 +'s = ''''  ' +s)
    else Memo1.Append( '<SF_STR_SOFTWARE> ' +#9 +s);

  Memo1.Append( '------------------------------------');
end;


procedure TForm1.ButtonTestReadClick(Sender: TObject);
var
  BufFrames : sf_count_t;
  OutFrames : sf_count_t;
  NumBytes  : sf_count_t;
  arr       : array[0..2047] of ctypes.cshort;  //1023 for stereo only !
  res       : integer;
  i         : integer;
begin
  i:= 0;
  res:= sf_format_check( SFinfo);
  Memo1.Append( '------------------------------------');
  Memo1.Append( 'sf_format_check result:  $' +IntToHex( res, 8));

  BufFrames:= Length( arr);
  repeat
    OutFrames:= sf_read_short( SFhandle, @arr[0], BufFrames);
    //Memo1.Append( '> ' +IntToStr(i) +'BufFrames:=  ' +IntToStr( BufFrames));
    //Memo1.Append( '> ' +IntToStr(i) +'Outframes:=  ' +IntToStr( Outframes));
    inc( i);
  until OutFrames < BufFrames;
  Memo1.Append( '> *** reading ' +IntToStr(i) +' Buffers with sf_read_short ***');
  Memo1.Append( '> last buffer - BufFrames:=  ' +IntToStr( BufFrames));
  Memo1.Append( '> last buffer - Outframes:=  ' +IntToStr( Outframes));
  Memo1.Append( '------------------------------------');

  // here we must reset the "file pointer" to read from start of DATA :
  sf_seek( SFhandle, 0, SEEK_SET);
  // read raw data :
  NumBytes:= length( arr);
  NumBytes:= sf_read_raw( SFhandle, @arr[0], NumBytes);
  Memo1.Append( '> *** reading ' +IntToStr(NumBytes) +' Buffers with sf_read_raw ***');
  Memo1.Append( '> read raw data :  NumBytes =  ' +IntToStr( NumBytes));
end;


procedure TForm1.ButtonLibVersionClick(Sender: TObject);
begin
  WriteStringToMemo( 'Library Version =  ' +sf_LibSndVersionStr);
end;


procedure TForm1.ButtonLogStrClick(Sender: TObject);
begin
  WriteStringToMemo( 'Log Infos:');
  Memo1.Append( sf_LibSndLogStr( SFhandle));
  if sf_error( SFhandle) <> 0 then WriteStringToMemo( sf_strerror( SFhandle));
end;


procedure TForm1.ButtonMaxSampleClick(Sender: TObject);
begin
  if length( Edit1.Caption) > 0
    then Memo1.Append( 'Maximum Signal Value =  '
                            +FloatToStr( sf_FileGetMaxSample( SFhandle)));
  if sf_error( SFhandle) <> 0 then WriteStringToMemo( sf_strerror( SFhandle));
end;


procedure TForm1.ButtonMaxNormalizedClick(Sender: TObject);
begin
  if length( Edit1.Caption) > 0
    then Memo1.Append( 'Maximum Normalized Signal Value =  '
                            +FloatToStr( sf_FileGetNormalisedMaxSample( SFhandle)));
  if sf_error( SFhandle) <> 0 then WriteStringToMemo( sf_strerror( SFhandle));
end;


procedure TForm1.ButtonMaxAllChannelsClick(Sender: TObject);
begin
  if SFhandle <> NIL
    then Memo1.Append( 'Maximum Signal Value all channels =  '
                            +FloatToStr( sf_FileGetMaxAllChannels( SFhandle,
                                                                   SFinfo.channels)));
  if sf_error( SFhandle) <> 0 then WriteStringToMemo( sf_strerror( SFhandle));
end;


procedure TForm1.ButtonGetBroadcastChkClick(Sender: TObject);
var
  sArr : TStringList;
  i    : Integer;
begin
  if Length( Edit1.Caption) > 0 then
    begin
      sArr:= TStringList.Create;
      sf_FileGetBroadcastChunk( SFhandle, sArr);
      Memo1.Visible:= FALSE;
      for i:= 0 to sArr.Count -1
        do Memo1.Append( sArr[i]);
      Memo1.Visible:= TRUE;
      sArr.Free;

      if sf_error( SFhandle) <> 0 then WriteStringToMemo( sf_strerror( SFhandle));
    end;
end;


// --- Subfunctions ------------------------------------------------------------
procedure TForm1.Show_SFinfo;
begin
  Memo1.Append( 'Info.format value   = ' +IntToStr( SFinfo.format));
  Memo1.Append( '   formatmajorstr   = ' +sf_formatmajorstr( SFinfo.format));
  Memo1.Append( '   formatsubtypestr = ' +sf_formatsubtypestr( SFinfo.format));
  Memo1.Append( 'Info.channels       = ' +IntToStr( SFinfo.channels));
  Memo1.Append( 'Info.samplerate     = ' +IntToStr( SFinfo.samplerate));
  Memo1.Append( 'Info.frames         = ' +IntToStr( SFinfo.frames));
  Memo1.Append( 'Info.sections       = ' +IntToStr( SFinfo.sections));
  Memo1.Append( 'Info.seekable       = ' +IntToStr( SFinfo.seekable));
  Memo1.Append( '');
end;


procedure TForm1.ActivateButtons( b: boolean);
begin
  ButtonOpenFile.Enabled:= NOT b;
  ButtonCloseFile.Enabled:= b;
  //ButtonLibVersion.Enabled:= b;  // no! ... always works ... !
  ButtonLogStr.Enabled:= b;
  ButtonMaxSample.Enabled:= b;
  ButtonMaxNormalized.Enabled:= b;
  ButtonMaxAllChannels.Enabled:= b;
  ButtonGetBroadcastChk.Enabled:= b;
  ButtonGetStrings.Enabled:= b;
  ButtonTestRead.Enabled:= b;
end;


procedure TForm1.WriteStringToMemo( s: String);
begin
  Memo1.Append( s);
  Memo1.Append( '');
end;


end.

