unit libsndfile;

{$mode objfpc}{$H+}

{*******************************************************************************
*                      Header file for  libSndFile.DLL
*
*                        latest changes:  2021-09-23
*******************************************************************************}

interface

uses
  Interfaces,
  //Windows,  not in use
  cTypes;


function LoadSndFileDll:boolean;
procedure UnLoadSndFileDll;



// -----------------------------------------------------------------------------
(*
 - in parts adapted from Translation for sndfile.h version 1.0.17 by Ido Kanner idokan at gmail dot com
*)
// Copyright (C) 1999-2006 Erik de Castro Lopo <erikd@mega-nerd.com>
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation; either version 2.1 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
// sndfile.h -- system-wide definitions
//
// API documentation is in the doc/ directory of the source code tarball
// and at http://www.mega-nerd.com/libsndfile/api.html.
//
//

//* This is the version 1.0.X header file. */

// The following file types can be read and written.
// A file type would consist of a major type (ie SF_FORMAT_WAV) bitwise
// ORed with a minor type (ie SF_FORMAT_PCM). SF_FORMAT_TYPEMASK and
// SF_FORMAT_SUBMASK can be used to separate the major and minor file types.
// -----------------------------------------------------------------------------


const
  //* Major formats. *//
  SF_FORMAT_WAV   = $010000;       // Microsoft WAV format (little endian default).
  SF_FORMAT_AIFF  = $020000;       // Apple/SGI AIFF format (big endian).
  SF_FORMAT_AU    = $030000;       // Sun/NeXT AU format (big endian).
  SF_FORMAT_RAW   = $040000;       // RAW PCM data.
  SF_FORMAT_PAF   = $050000;       // Ensoniq PARIS file format.
  SF_FORMAT_SVX   = $060000;       // Amiga IFF / SVX8 / SV16 format.
  SF_FORMAT_NIST  = $070000;       // Sphere NIST format.
  SF_FORMAT_VOC	  = $080000;       // VOC files.
  SF_FORMAT_IRCAM = $0A0000;       // Berkeley/IRCAM/CARL
  SF_FORMAT_W64   = $0B0000;       // Sonic Foundry's 64 bit RIFF/WAV
  SF_FORMAT_MAT4  = $0C0000;       // Matlab (tm) V4.2 / GNU Octave 2.0
  SF_FORMAT_MAT5  = $0D0000;       // Matlab (tm) V5.0 / GNU Octave 2.1
  SF_FORMAT_PVF   = $0E0000;       // Portable Voice Format
  SF_FORMAT_XI    = $0F0000;       // Fasttracker 2 Extended Instrument
  SF_FORMAT_HTK   = $100000;       // HMM Tool Kit format
  SF_FORMAT_SDS   = $110000;       // Midi Sample Dump Standard
  SF_FORMAT_AVR   = $120000;       // Audio Visual Research
  SF_FORMAT_WAVEX = $130000;       // MS WAVE with WAVEFORMATEX
  SF_FORMAT_SD2   = $160000;       // Sound Designer 2
  SF_FORMAT_FLAC  = $170000;       // FLAC lossless file format
  SF_FORMAT_CAF   = $180000;       // Core Audio File format

  //Subtypes from here on.
  SF_FORMAT_PCM_S8    = $0001;     // Signed 8 bit data
  SF_FORMAT_PCM_16    = $0002;     // Signed 16 bit data
  SF_FORMAT_PCM_24    = $0003;     // Signed 24 bit data
  SF_FORMAT_PCM_32    = $0004;     // Signed 32 bit data

  SF_FORMAT_PCM_U8    = $0005;     // Unsigned 8 bit data (WAV and RAW only)

  SF_FORMAT_FLOAT     = $0006;     // 32 bit float data
  SF_FORMAT_DOUBLE    = $0007;     // 64 bit float data

  SF_FORMAT_ULAW      = $0010;     // U-Law encoded.
  SF_FORMAT_ALAW      = $0011;     // A-Law encoded.
  SF_FORMAT_IMA_ADPCM = $0012;     // IMA ADPCM.
  SF_FORMAT_MS_ADPCM  = $0013;     // Microsoft ADPCM.

  SF_FORMAT_GSM610    = $0020;     // GSM 6.10 encoding.
  SF_FORMAT_VOX_ADPCM = $0021;     // OKI / Dialogix ADPCM

  SF_FORMAT_G721_32   = $0030;     // 32kbs G721 ADPCM encoding.
  SF_FORMAT_G723_24   = $0031;     // 24kbs G723 ADPCM encoding.
  SF_FORMAT_G723_40   = $0032;     // 40kbs G723 ADPCM encoding.

  SF_FORMAT_DWVW_12   = $0040;     // 12 bit Delta Width Variable Word encoding.
  SF_FORMAT_DWVW_16   = $0041;     // 16 bit Delta Width Variable Word encoding.
  SF_FORMAT_DWVW_24   = $0042;     // 24 bit Delta Width Variable Word encoding.
  SF_FORMAT_DWVW_N    = $0043;     // N bit Delta Width Variable Word encoding.

  SF_FORMAT_DPCM_8    = $0050;     // 8 bit differential PCM (XI only)
  SF_FORMAT_DPCM_16   = $0051;     // 16 bit differential PCM (XI only)


const
  //* Endian-ness options. *//
  SF_ENDIAN_FILE     = $00000000;  // Default file endian-ness.
  SF_ENDIAN_LITTLE   = $10000000;  // Force little endian-ness.
  SF_ENDIAN_BIG      = $20000000;  // Force big endian-ness.
  SF_ENDIAN_CPU      = $30000000;  // Force CPU endian-ness.

  SF_FORMAT_SUBMASK  = $0000FFFF;
  SF_FORMAT_TYPEMASK = $0FFF0000;
  SF_FORMAT_ENDMASK  = $30000000;


// The following are the valid command numbers for the sf_command()
// interface.  The use of these commands is documented in the file
// command.html in the doc directory of the source code distribution.
//
const
  SFC_GET_LIB_VERSION            = $1000;
  SFC_GET_LOG_INFO               = $1001;

  SFC_GET_NORM_DOUBLE            = $1010;
  SFC_GET_NORM_FLOAT             = $1011;
  SFC_SET_NORM_DOUBLE            = $1012;
  SFC_SET_NORM_FLOAT             = $1013;
  SFC_SET_SCALE_FLOAT_INT_READ   = $1014;

  SFC_GET_SIMPLE_FORMAT_COUNT    = $1020;
  SFC_GET_SIMPLE_FORMAT          = $1021;

  SFC_GET_FORMAT_INFO            = $1028;

  SFC_GET_FORMAT_MAJOR_COUNT     = $1030;
  SFC_GET_FORMAT_MAJOR           = $1031;
  SFC_GET_FORMAT_SUBTYPE_COUNT   = $1032;
  SFC_GET_FORMAT_SUBTYPE         = $1033;

  SFC_CALC_SIGNAL_MAX            = $1040;
  SFC_CALC_NORM_SIGNAL_MAX       = $1041;
  SFC_CALC_MAX_ALL_CHANNELS      = $1042;
  SFC_CALC_NORM_MAX_ALL_CHANNELS = $1043;
  SFC_GET_SIGNAL_MAX             = $1044;
  SFC_GET_MAX_ALL_CHANNELS       = $1045;

  SFC_SET_ADD_PEAK_CHUNK         = $1050;

  SFC_UPDATE_HEADER_NOW          = $1060;
  SFC_SET_UPDATE_HEADER_AUTO     = $1061;

  SFC_FILE_TRUNCATE	         = $1080;

  SFC_SET_RAW_START_OFFSET       = $1090;

  SFC_SET_DITHER_ON_WRITE        = $10A0;
  SFC_SET_DITHER_ON_READ         = $10A1;

  SFC_GET_DITHER_INFO_COUNT      = $10A2;
  SFC_GET_DITHER_INFO            = $10A3;

  SFC_GET_EMBED_FILE_INFO        = $10B0;

  SFC_SET_CLIPPING               = $10C0;
  SFC_GET_CLIPPING               = $10C1;

  SFC_GET_INSTRUMENT             = $10D0;
  SFC_SET_INSTRUMENT             = $10D1;

  SFC_GET_LOOP_INFO              = $10E0;

  SFC_GET_BROADCAST_INFO         = $10F0;
  SFC_SET_BROADCAST_INFO         = $10F1;

  // Following commands for testing only.
  SFC_TEST_IEEE_FLOAT_REPLACE	 = $6001;


// SFC_SET_ADD_* values are deprecated and will disappear at some
// time in the future. They are guaranteed to be here up to and
// including version 1.0.8 to avoid breakage of existng software.
// They currently do nothing and will continue to do nothing.
//
const
  SFC_SET_ADD_DITHER_ON_WRITE		= $1070;
  SFC_SET_ADD_DITHER_ON_READ		= $1071;


// String types that can be set and read from files. Not all file types
// support this and even the file types which support one, may not support
// all string types.
//
const
  SF_STR_TITLE     = $01;
  SF_STR_COPYRIGHT = $02;
  SF_STR_SOFTWARE  = $03;
  SF_STR_ARTIST    = $04;
  SF_STR_COMMENT   = $05;
  SF_STR_DATE      = $06;


// Use the following as the start and end index when doing metadata transcoding.
//
const
  SF_STR_FIRST     = SF_STR_TITLE;
  SF_STR_LAST      = SF_STR_DATE;

const
  // True and false
  SF_FALSE = 0;
  SF_TRUE  = 1;

const
  // Modes for opening files.
  SFM_READ  = $10;
  SFM_WRITE = $20;
  SFM_RDWR  = $30;

// Public error values. These are guaranteed to remain unchanged for the duration
// of the library major version number.
// There are also a large number of private error numbers which are internal to
// the library which can change at any time.
//
const
  SF_ERR_NO_ERROR             = 0;
  SF_ERR_UNRECOGNISED_FORMAT  = 1;
  SF_ERR_SYSTEM               = 2;
  SF_ERR_MALFORMED_FILE       = 3;
  SF_ERR_UNSUPPORTED_ENCODING = 4;
  SF_ERR_NOLIB                = 90112;   // added, is not in original sources ..


// A SNDFILE* pointer can be passed around much like stdio.h's FILE* pointer.
type
  TSNDFILE_HANDLE = pointer;  // this is not a usual pointer, more like a THandle ..
                              // => we never access members of the internal
                              //    structure where the pointer points to !
                              //    Everything is managed by the DLL internally !!!


// The following typedef is system specific and is defined when libsndfile is.
// compiled. sf_count_t can be one of loff_t (Linux), off_t (*BSD),
// off64_t (Solaris), __int64_t (Win32) etc.
//
type
  sf_count_t = int64;


const
  SF_COUNT_MAX = $7FFFFFFFFFFFFFFF;


// A pointer to a SF_INFO structure is passed to sf_open_read () and filled in.
// On write, the SF_INFO structure is filled in by the user and passed into
// sf_open_write ().
//
type
  PSF_INFO = ^TSF_INFO;
  TSF_INFO = record
	frames     : sf_count_t;//* Used to be called samples.  Changed to avoid confusion. */
	samplerate : integer;
	channels   : integer;
	format     : integer;
	sections   : integer;
	seekable   : integer;
  end;


// The SF_FORMAT_INFO struct is used to retrieve information about the sound
// file formats libsndfile supports using the sf_command () interface.
//
// Using this interface will allow applications to support new file formats
// and encoding types when libsndfile is upgraded, without requiring
// re-compilation of the application.
//
// Please consult the libsndfile documentation (particularly the information
// on the sf_command () interface) for examples of its use.
//
type
  SF_FORMAT_INFO = packed record
    format    : integer;
    name      : pchar;
    extension : pchar;
  end;


// Enums and typedefs for adding dither on read and write.
// See the html documentation for sf_command(), SFC_SET_DITHER_ON_WRITE
// and SFC_SET_DITHER_ON_READ.
//
const
  SFD_DEFAULT_LEVEL  = 0;
  SFD_CUSTOM_LEVEL   = $40000000;

  SFD_NO_DITHER      = 500;
  SFD_WHITE          = 501;
  SFD_TRIANGULAR_PDF = 502;


type
  SF_DITHER_INFO = packed record
    _type : integer;
    level : double;
    name  : pchar;
  end;


// Struct used to retrieve information about a file embedded within a
// larger file. See SFC_GET_EMBED_FILE_INFO.
//
type
  SF_EMBED_FILE_INFO = packed record
    offset : sf_count_t;
    length : sf_count_t;
  end;

// Structs used to retrieve music sample information from a file.
//
const
// The loop mode field in SF_INSTRUMENT will be one of the following.
  SF_LOOP_NONE        = 800;
  SF_LOOP_FORWARD     = 801;
  SF_LOOP_BACKWARD    = 802;
  SF_LOOP_ALTERNATING = 803;

type
  SF_INSTRUMENT = packed record
    gain        : integer;
    basenote    : shortint;
    detune      : shortint;
    velocity_lo : shortint;
    velocity_hi : shortint;
    loop_count  : integer;
    loops       : array[0..15] of record
                                    mode  : integer;
                                    start : longword;
                                    end_  : longword;
                                    count : longword;
                                  end;
end;

// Struct used to retrieve loop information from a file.*/
type
  SF_LOOP_INFO = packed record
    time_sig_num : word;     //* any positive integer    >0  */
    time_sig_den : word;     //* any positive power of 2 >0  */
    loop_mode    : integer;  //* see SF_LOOP enum            */
    num_beats    : integer;  //* this is NOT the amount of quarter notes !!!*/
    			     //* a full bar of 4/4 is 4 beats */
    			     //* a full bar of 7/8 is 7 beats */

    bpm          : single;   //* suggestion, as it can be calculated using other fields:*/
    			     //* file's lenght, file's sampleRate and our time_sig_den*/
    			     //* -> bpms are always the amount of _quarter notes_ per minute */

    root_key     : integer;  //* MIDI note, or -1 for None */
    future       : array[0..5] of integer;
  end;


// Struct used to retrieve broadcast (EBU) information from a file.
// Strongly (!) based on EBU "bext" chunk format used in Broadcast WAVE.
//
type
  PSF_BROADCAST_INFO = ^TSF_BROADCAST_INFO;
  TSF_BROADCAST_INFO = record
    description          : array[0..255] of char;
    originator           : array[0..31]  of char;
    originator_reference : array[0..31]  of char;
    origination_date     : array[0..9]   of char;
    origination_time     : array[0..7]   of char;
    time_reference_low   : ctypes.cuint;
    time_reference_high  : ctypes.cuint;
    version              : ctypes.cshort;
    umid                 : array[0..63]  of char;
    reserved             : array[0..189] of char;
    coding_history_size  : ctypes.cuint;
    coding_history       : array[0..255] of char;
  end;


type
  Tsf_vio_get_filelen = function (user_date : pointer) : sf_count_t;           cdecl;

  Tsf_vio_seek        = function (offest : sf_count_t; whence : ctypes.cint;
                                   user_date : pointer) : sf_count_t;           cdecl;

  Tsf_vio_read        = function (ptr : Pointer; count : sf_count_t;
                                   user_date : pointer) : sf_count_t;           cdecl;

  Tsf_vio_write       = function (ptr : Pointer; count : sf_count_t;
                                   user_date : pointer) : sf_count_t;           cdecl;

  Tsf_vio_tell        = function (user_data : pointer) : sf_count_t;           cdecl;


  PSF_VIRTUAL_IO = ^TSF_VIRTUAL_IO;
  TSF_VIRTUAL_IO = record
                     get_filelen : Tsf_vio_get_filelen;
                     seek        : Tsf_vio_seek;
                     read        : Tsf_vio_read;
                     write       : Tsf_vio_write;
                     tell        : Tsf_vio_tell;
                   end;


type
  // Open the specified file for read, write or both. On error, this will
  // return a NULL pointer. To find the error number, pass a NULL SNDFILE
  // to sf_perror () or sf_error_str ().
  // All calls to sf_open() should be matched with a call to sf_close().
  Tlsf_open_native = function( path:pchar; mode:integer; sfinfo:PSF_INFO):TSNDFILE_HANDLE; cdecl;

  // Use the existing file descriptor to create a SNDFILE object. If close_desc
  // is TRUE, the file descriptor will be closed when sf_close() is called. If
  // it is FALSE, the descritor will not be closed.
  // When passed a descriptor like this, the library will assume that the start
  // of file header is at the current file offset. This allows sound files within
  // larger container files to be read and/or written.
  // On error, this will return a NULL pointer. To find the error number, pass a
  // NULL SNDFILE to sf_perror () or sf_error_str ().
  // All calls to sf_open_fd() should be matched with a call to sf_close().
  Tlsf_open_fd = function( fd, mode:integer; sfinfo:PSF_INFO; close_desc:integer):TSNDFILE_HANDLE; cdecl;

  // sf_error () returns a error number which can be translated to a text string
  // using sf_error_number().
  Tlsf_error = function( sndfile:TSNDFILE_HANDLE):integer; cdecl;

  // sf_strerror () returns to the caller a pointer to the current error message
  // for the given SNDFILE.
  Tlsf_strerror = function( sndfile:TSNDFILE_HANDLE):pchar; cdecl;

  // sf_error_number () allows the retrieval of the error string for each
  // internal error number.
  Tlsf_error_number = function( errnum:integer):pchar; cdecl;

  // The following three error functions are deprecated but they will remain
  // in the library for the forseeable future. The function sf_strerror()
  // should be used in their place.
  Tlsf_perror    = function( sndfile:TSNDFILE_HANDLE):integer; cdecl;
  Tlsf_error_str = function( sndfile:TSNDFILE_HANDLE; str:pchar; len:integer):integer; cdecl;

  // Return TRUE if fields of the SF_INFO struct are a valid combination of values. */
  Tlsf_command = function( sndfile:TSNDFILE_HANDLE; command:integer; data:pointer;
                           datasize:integer):integer; cdecl;

  // Return TRUE if fields of the SF_INFO struct are a valid combination of values. */
  Tlsf_format_check = function( info:PSF_INFO):integer; cdecl;

  // Seek within the waveform data chunk of the SNDFILE. sf_seek ()
  // uses the same values for whence (SEEK_SET, SEEK_CUR and SEEK_END)
  // as stdio.h function fseek ().
  // An offset of zero with whence set to SEEK_SET will position the
  // read / write pointer to the first data sample.
  // On success sf_seek returns the current position in (multi-channel)
  // samples from the start of the file.
  // Please see the libsndfile documentation for moving the read pointer
  // separately from the write pointer on files open in mode SFM_RDWR.
  // On error all of these functions return -1.
  //
  // The following CONST values originally are NOT in libsndfile.pas:
  const SEEK_SET  = 0;       //* seek relative to beginning of file */
  const SEEK_CUR  = 1;       //* seek relative to current file position */
  const SEEK_END  = 2;       //* seek relative to end of file */
  const SEEK_DATA = 3;       //* seek to the next data */
  const SEEK_HOLE = 4;       //* seek to the next hole */
  const SEEK_MAX = SEEK_HOLE;

type
  Tlsf_seek = function( sndfile:TSNDFILE_HANDLE; frames:sf_count_t;
                       whence:integer):sf_count_t; cdecl;

  // Functions for retrieving and setting string data within sound files.
  // Not all file types support this features; AIFF and WAV do. For both
  // functions, the str_type parameter must be one of the SF_STR_* values
  // defined above.
  // On error, sf_set_string() returns non-zero while sf_get_string()
  // returns NULL.
  Tlsf_set_string = function( sndfile:TSNDFILE_HANDLE; str_type:integer;
                             str:pchar):integer; cdecl;
  Tlsf_get_string = function( sndfile:TSNDFILE_HANDLE;
                             str_type:integer):pchar; cdecl;

  // Functions for reading/writing the waveform data of a sound file.
  Tlsf_read_raw  = function( sndfile:TSNDFILE_HANDLE; ptr:pointer;
                            bytes:sf_count_t):sf_count_t; cdecl;
  Tlsf_write_raw = function( sndfile:TSNDFILE_HANDLE; ptr:pointer;
                            bytes:sf_count_t):sf_count_t; cdecl;

  // ---------------------------------------------------------------------------
  // ---------------------------------------------------------------------------
  // 2021-09-21 - up to here merged with other file ... ------------------------
  // ---------------------------------------------------------------------------
  // ---------------------------------------------------------------------------

  // Functions for reading and writing the data chunk in terms of frames.
  // The number of items actually read/written = frames * number of channels.
  //     sf_xxxx_raw		read/writes the raw data bytes from/to the file
  //     sf_xxxx_short	passes data in the native short format
  //     sf_xxxx_int		passes data in the native int format
  //     sf_xxxx_float	passes data in the native float format
  //     sf_xxxx_double	passes data in the native double format
  // All of these read/write function return number of frames read/written.
  //
  Tlsf_readf_short   = function( sndfile:TSNDFILE_HANDLE; ptr:psmallint; frames:sf_count_t):sf_count_t; cdecl;
  Tlsf_writef_short  = function( sndfile:TSNDFILE_HANDLE; ptr:psmallint; frames:sf_count_t):sf_count_t; cdecl;
  Tlsf_readf_int     = function( sndfile:TSNDFILE_HANDLE; ptr:pinteger;  frames:sf_count_t):sf_count_t; cdecl;
  Tlsf_writef_int    = function( sndfile:TSNDFILE_HANDLE; ptr:pinteger;  frames:sf_count_t):sf_count_t; cdecl;
  Tlsf_readf_float   = function( sndfile:TSNDFILE_HANDLE; ptr:psingle;   frames:sf_count_t):sf_count_t; cdecl;
  Tlsf_writef_float  = function( sndfile:TSNDFILE_HANDLE; ptr:psingle;   frames:sf_count_t):sf_count_t; cdecl;
  Tlsf_readf_double  = function( sndfile:TSNDFILE_HANDLE; ptr:pdouble;   frames:sf_count_t):sf_count_t; cdecl;
  Tlsf_writef_double = function( sndfile:TSNDFILE_HANDLE; ptr:pdouble;   frames:sf_count_t):sf_count_t; cdecl;

  ///* Functions for reading and writing the data chunk in terms of items.
  //** Otherwise similar to above.
  //** All of these read/write function return number of items read/written.
  Tlsf_read_short    = function( sndfile:TSNDFILE_HANDLE; ptr:psmallint; bytes:sf_count_t):sf_count_t; cdecl;
  Tlsf_write_short   = function( sndfile:TSNDFILE_HANDLE; ptr:psmallint; bytes:sf_count_t):sf_count_t; cdecl;
  Tlsf_read_int      = function( sndfile:TSNDFILE_HANDLE; ptr:pinteger;  bytes:sf_count_t):sf_count_t; cdecl;
  Tlsf_write_int     = function( sndfile:TSNDFILE_HANDLE; ptr:pinteger;  bytes:sf_count_t):sf_count_t; cdecl;
  Tlsf_read_float    = function( sndfile:TSNDFILE_HANDLE; ptr:psingle;   bytes:sf_count_t):sf_count_t; cdecl;
  Tlsf_write_float   = function( sndfile:TSNDFILE_HANDLE; ptr:psingle;   bytes:sf_count_t):sf_count_t; cdecl;
  Tlsf_read_double   = function( sndfile:TSNDFILE_HANDLE; ptr:pdouble;   bytes:sf_count_t):sf_count_t; cdecl;
  Tlsf_write_double  = function( sndfile:TSNDFILE_HANDLE; ptr:pdouble;   bytes:sf_count_t):sf_count_t; cdecl;

  // Close the SNDFILE and clean up all memory allocations
  // associated with this file. Returns 0 on success, or an error number.
  Tlsf_close = function( sndfile:TSNDFILE_HANDLE):integer; cdecl;


var
  lsf_open_native   : Tlsf_open_native;  // works with pchar instead of Pascal's string
  lsf_open_fd       : Tlsf_open_fd;
  lsf_error         : Tlsf_error;
  lsf_strerror      : Tlsf_strerror;
  lsf_error_number  : Tlsf_error_number;
  lsf_perror        : Tlsf_perror;
  lsf_error_str     : Tlsf_error_str;
  lsf_command       : Tlsf_command;
  lsf_format_check  : Tlsf_format_check;
  lsf_seek          : Tlsf_seek;
  lsf_set_string    : Tlsf_set_string;
  lsf_get_string    : Tlsf_get_string;
  lsf_read_raw      : Tlsf_read_raw;
  lsf_write_raw     : Tlsf_write_raw;
  lsf_readf_short   : Tlsf_readf_short;
  lsf_writef_short  : Tlsf_writef_short;
  lsf_readf_int     : Tlsf_readf_int;
  lsf_writef_int    : Tlsf_writef_int;
  lsf_readf_float   : Tlsf_readf_float;
  lsf_writef_float  : Tlsf_writef_float;
  lsf_readf_double  : Tlsf_readf_double;
  lsf_writef_double : Tlsf_writef_double;
  lsf_read_short    : Tlsf_read_short;
  lsf_write_short   : Tlsf_write_short;
  lsf_read_int      : Tlsf_read_int;
  lsf_write_int     : Tlsf_write_int;
  lsf_read_float    : Tlsf_read_float;
  lsf_write_float   : Tlsf_write_float;
  lsf_read_double   : Tlsf_read_double;
  lsf_write_double  : Tlsf_write_double;
  lsf_close         : Tlsf_close;


function sf_open         (path:string; mode:integer; var sfinfo:TSF_INFO):TSNDFILE_HANDLE ;
function sf_open_fd      (fd, mode:integer; var sfInfo:TSF_INFO; close_desc:integer):TSNDFILE_HANDLE;
function sf_error        (sndfile:TSNDFILE_HANDLE):integer;
function sf_strerror     (sndfile:TSNDFILE_HANDLE):pchar;
function sf_error_number (errnum:integer):pchar;
function sf_perror       (sndfile:TSNDFILE_HANDLE):integer;
function sf_error_str    (sndfile:TSNDFILE_HANDLE; str:pchar; len:integer):integer;
function sf_command      (sndfile:TSNDFILE_HANDLE; command:integer; data:pointer; datasize:integer) :integer;  overload;
function sf_command      (sndfile:TSNDFILE_HANDLE; command:integer; var data:Double;
                          datasize:integer):integer; overload;
function sf_command      (sndfile:TSNDFILE_HANDLE; command:integer; p:pchar; //var data: Array of Char; => causes Warning: cdecl'ared functions have no high parameter
                          datasize:integer):integer; overload;
function sf_command      (sndfile:TSNDFILE_HANDLE; command:integer; var data: TSF_BROADCAST_INFO;
                          datasize:integer):integer; overload;
function sf_format_check (info:TSF_INFO):integer;
function sf_seek         (sndfile:TSNDFILE_HANDLE; frames:sf_count_t;whence:integer):sf_count_t;
function sf_set_string   (sndfile:TSNDFILE_HANDLE;str_type:integer; str:string):integer;
function sf_get_string   (sndfile:TSNDFILE_HANDLE; str_type:integer):string;
function sf_read_raw     (sndfile:TSNDFILE_HANDLE;ptr:pointer;bytes:sf_count_t):sf_count_t;
function sf_write_raw    (sndfile:TSNDFILE_HANDLE;ptr:pointer;bytes:sf_count_t):sf_count_t;
function sf_readf_short  (sndfile:TSNDFILE_HANDLE; ptr:psmallint; frames:sf_count_t):sf_count_t;
function sf_writef_short (sndfile:TSNDFILE_HANDLE; ptr:psmallint; frames:sf_count_t):sf_count_t;
function sf_readf_int    (sndfile:TSNDFILE_HANDLE; ptr:pinteger;  frames:sf_count_t):sf_count_t;
function sf_writef_int   (sndfile:TSNDFILE_HANDLE; ptr:pinteger;  frames:sf_count_t):sf_count_t;
function sf_readf_float  (sndfile:TSNDFILE_HANDLE; ptr:psingle;   frames:sf_count_t):sf_count_t;
function sf_writef_float (sndfile:TSNDFILE_HANDLE; ptr:psingle;   frames:sf_count_t):sf_count_t;
function sf_readf_double (sndfile:TSNDFILE_HANDLE; ptr:pdouble;   frames:sf_count_t):sf_count_t;
function sf_writef_double(sndfile:TSNDFILE_HANDLE; ptr:pdouble;   frames:sf_count_t):sf_count_t;
function sf_read_short   (sndfile:TSNDFILE_HANDLE; ptr:psmallint; bytes:sf_count_t):sf_count_t;
function sf_write_short  (sndfile:TSNDFILE_HANDLE; ptr:psmallint; bytes:sf_count_t):sf_count_t;
function sf_read_int     (sndfile:TSNDFILE_HANDLE; ptr:pinteger;  bytes:sf_count_t):sf_count_t;
function sf_write_int    (sndfile:TSNDFILE_HANDLE; ptr:pinteger;  bytes:sf_count_t):sf_count_t;
function sf_read_float   (sndfile:TSNDFILE_HANDLE; ptr:psingle;   bytes:sf_count_t):sf_count_t;
function sf_write_float  (sndfile:TSNDFILE_HANDLE; ptr:psingle;   bytes:sf_count_t):sf_count_t;
function sf_read_double  (sndfile:TSNDFILE_HANDLE; ptr:pdouble;   bytes:sf_count_t):sf_count_t;
function sf_write_double (sndfile:TSNDFILE_HANDLE; ptr:pdouble;   bytes:sf_count_t):sf_count_t;
function sf_close        (sndfile:TSNDFILE_HANDLE):integer;



function SndFileDllLoaded : Boolean;


implementation


function sf_open( path : string; mode:integer; var sfInfo:TSF_INFO):TSNDFILE_HANDLE;
begin
  if SndFileDllLoaded
    then Result:= lsf_open_native( pChar( path), mode, @sfinfo)
    else Result:= nil;
end;

function sf_open_fd( fd, mode:integer; var sfInfo:TSF_INFO;
                     close_desc:integer):TSNDFILE_HANDLE;
begin
  if SndFileDllLoaded
    then Result:= lsf_open_fd( fd, mode, @sfinfo, close_desc)
    else Result:= nil;
end;

function sf_error( sndfile:TSNDFILE_HANDLE):integer;
begin
  if SndFileDllLoaded
    then Result:= lsf_error( sndfile)
    else Result:= SF_ERR_NOLIB;
end;

function sf_strerror( sndfile:TSNDFILE_HANDLE):pchar;
begin
  if SndFileDllLoaded
    then Result:= lsf_strerror( sndfile)
    else Result:= nil;
end;

function sf_error_number( errnum:integer):pchar;
begin
  if SndFileDllLoaded
    then Result:= lsf_error_number( errnum)
    else Result:= nil;
end;

function sf_perror( sndfile:TSNDFILE_HANDLE):integer;
begin
  if SndFileDllLoaded
    then Result:= lsf_perror( sndfile)
    else Result:= SF_ERR_NOLIB;
end;

//ToDo: somehow replace str:pchar with string
function sf_error_str( sndfile:TSNDFILE_HANDLE; str:pchar; len:integer):integer;
begin
  if SndFileDllLoaded
    then Result:= lsf_error_str( sndfile, str, len)
    else Result:= SF_ERR_NOLIB;
end;


function sf_command( sndfile:TSNDFILE_HANDLE; command:integer; data:pointer;
                     datasize:integer):integer; overload;
begin
  if SndFileDllLoaded
    then Result:= lsf_command( sndfile, command, data, datasize)
    else Result:= SF_ERR_NOLIB;
end;

function sf_command( sndfile:TSNDFILE_HANDLE; command:integer; var data:Double;
                     datasize:integer):integer; overload;
begin
  if SndFileDllLoaded
    then Result:= lsf_command( sndfile, command, pointer( @data), datasize)
    else Result:= SF_ERR_NOLIB;                  // not sure if this is a valid type cast !
end;

// Passing an open array (var data: Array of Char)
// is not supported with cdecl or cppdecl.
// In PASCAL open arrays are realized internally by passing a pointer
// to the start of the array and an (implicit) high parameter
// which lets the compiler deduce the length of the array.
function sf_command( sndfile:TSNDFILE_HANDLE; command:integer; p:pchar; //var data: Array of Char; => causes Warning: cdecl'ared functions have no high parameter
                     datasize:integer):integer; overload;
begin
  if SndFileDllLoaded
    then Result:= lsf_command( sndfile, command, p, datasize)
    else Result:= SF_ERR_NOLIB;                  // not sure if this is a valid type cast !
end;

function sf_command( sndfile:TSNDFILE_HANDLE; command:integer; var data: TSF_BROADCAST_INFO;
                     datasize:integer):integer; overload;
begin
  if SndFileDllLoaded
    then Result:= lsf_command( sndfile, command, pointer( @data), datasize)
    else Result:= SF_ERR_NOLIB;                  // not sure if this is a valid type cast !
end;


function sf_format_check( info:TSF_INFO):integer;
begin
  if SndFileDllLoaded
    then Result:= lsf_format_check( @info)
    else Result:= SF_ERR_NOLIB;
end;

function sf_seek( sndfile:TSNDFILE_HANDLE; frames:sf_count_t; whence:integer):sf_count_t;
begin
  if SndFileDllLoaded
    then Result:= lsf_seek( sndfile, frames, whence)
    else Result:= SF_ERR_NOLIB;
end;

function sf_set_string( sndfile:TSNDFILE_HANDLE; str_type:integer; str:string):integer;
begin
  if SndFileDllLoaded
    then Result:= lsf_set_string( sndfile, str_type, pChar( str))
    else Result:= SF_ERR_NOLIB;
end;

function sf_get_string( sndfile:TSNDFILE_HANDLE; str_type:integer):string;
begin
  if SndFileDllLoaded
    then Result:= string( lsf_get_string( sndfile, str_type))
    else Result:= '';
end;

function sf_read_raw( sndfile:TSNDFILE_HANDLE; ptr:pointer;
                      bytes:sf_count_t):sf_count_t;
begin
  if SndFileDllLoaded
    then Result:= lsf_read_raw( sndfile, ptr, bytes)
    else Result:= SF_ERR_NOLIB;
end;

function sf_write_raw( sndfile:TSNDFILE_HANDLE; ptr:pointer;
                       bytes:sf_count_t):sf_count_t;
begin
  if SndFileDllLoaded
    then Result:= lsf_write_raw( sndfile, ptr, bytes)
    else Result:= SF_ERR_NOLIB;
end;

function sf_readf_short( sndfile:TSNDFILE_HANDLE; ptr:psmallint;
                         frames:sf_count_t):sf_count_t;
begin
  if SndFileDllLoaded
    then Result:= lsf_readf_short( sndfile, ptr, frames)
    else Result:= SF_ERR_NOLIB;
end;

function sf_writef_short( sndfile:TSNDFILE_HANDLE; ptr:psmallint;
                          frames:sf_count_t):sf_count_t;
begin
  if SndFileDllLoaded
    then Result:= lsf_writef_short( sndfile, ptr, frames)
    else Result:= SF_ERR_NOLIB;
end;

function sf_readf_int( sndfile:TSNDFILE_HANDLE; ptr:pinteger;
                       frames:sf_count_t):sf_count_t;
begin
  if SndFileDllLoaded
    then Result:= lsf_readf_int( sndfile, ptr, frames)
    else Result:= SF_ERR_NOLIB;
end;

function sf_writef_int( sndfile:TSNDFILE_HANDLE; ptr:pinteger;
                        frames:sf_count_t):sf_count_t;
begin
  if SndFileDllLoaded
    then Result:= lsf_writef_int( sndfile, ptr, frames)
    else Result:= SF_ERR_NOLIB;
end;

function sf_readf_float( sndfile:TSNDFILE_HANDLE; ptr:psingle;
                         frames:sf_count_t):sf_count_t;
begin
  if SndFileDllLoaded
    then Result:= lsf_readf_float( sndfile, ptr, frames)
    else Result:= SF_ERR_NOLIB;
end;

function sf_writef_float( sndfile:TSNDFILE_HANDLE; ptr:psingle;
                          frames:sf_count_t):sf_count_t;
begin
  if SndFileDllLoaded
    then Result:= lsf_writef_float( sndfile, ptr, frames)
    else Result:= SF_ERR_NOLIB;
end;

function sf_readf_double( sndfile:TSNDFILE_HANDLE; ptr:pdouble;
                          frames:sf_count_t):sf_count_t;
begin
  if SndFileDllLoaded
    then Result:= lsf_readf_double( sndfile, ptr, frames)
    else Result:= SF_ERR_NOLIB;
end;

function sf_writef_double( sndfile:TSNDFILE_HANDLE; ptr:pdouble;
                           frames:sf_count_t):sf_count_t;
begin
  if SndFileDllLoaded
    then Result:= lsf_writef_double( sndfile, ptr, frames)
    else Result:= SF_ERR_NOLIB;
end;

function sf_read_short( sndfile:TSNDFILE_HANDLE; ptr:psmallint;
                        bytes:sf_count_t):sf_count_t;
begin
  if SndFileDllLoaded
    then Result:= lsf_read_short( sndfile, ptr, bytes)
    else Result:= SF_ERR_NOLIB;
end;

function sf_write_short( sndfile:TSNDFILE_HANDLE; ptr:psmallint;
                         bytes:sf_count_t):sf_count_t;
begin
  if SndFileDllLoaded
    then Result:= lsf_write_short( sndfile, ptr, bytes)
    else Result:= SF_ERR_NOLIB;
end;

function sf_read_int( sndfile:TSNDFILE_HANDLE; ptr:pinteger;
                      bytes:sf_count_t):sf_count_t;
begin
  if SndFileDllLoaded
    then Result:= lsf_read_int( sndfile, ptr, bytes)
    else Result:= SF_ERR_NOLIB;
end;

function sf_write_int( sndfile:TSNDFILE_HANDLE; ptr:pinteger;
                       bytes:sf_count_t):sf_count_t;
begin
  if SndFileDllLoaded
    then Result:= lsf_write_int( sndfile, ptr, bytes)
    else Result:= SF_ERR_NOLIB;
end;

function sf_read_float( sndfile:TSNDFILE_HANDLE; ptr:psingle;
                        bytes:sf_count_t):sf_count_t;
begin
  if SndFileDllLoaded
    then Result:= lsf_read_float( sndfile, ptr, bytes)
    else Result:= SF_ERR_NOLIB;
end;

function sf_write_float( sndfile:TSNDFILE_HANDLE; ptr:psingle;
                         bytes:sf_count_t):sf_count_t;
begin
  if SndFileDllLoaded
    then Result:= lsf_write_float( sndfile, ptr, bytes)
    else Result:= SF_ERR_NOLIB;
end;

function sf_read_double( sndfile:TSNDFILE_HANDLE; ptr:pdouble;
                         bytes:sf_count_t):sf_count_t;
begin
  if SndFileDllLoaded
    then Result:= lsf_read_double( sndfile, ptr, bytes)
    else Result:= SF_ERR_NOLIB;
end;

function sf_write_double( sndfile:TSNDFILE_HANDLE; ptr:pdouble;
                          bytes:sf_count_t):sf_count_t;
begin
  if SndFileDllLoaded
    then Result:= lsf_write_double( sndfile, ptr, bytes)
    else Result:= SF_ERR_NOLIB;
end;

function sf_close( sndfile:TSNDFILE_HANDLE):integer;
begin
  if SndFileDllLoaded
    then Result:= lsf_close( sndfile)
    else Result:= SF_ERR_NOLIB;
end;


// -----------------------------------------------------------------------------
const LibName = 'libSndFile.DLL';

var SndFileDLL : THANDLE = 0;


function SndFileDllLoaded: Boolean;
begin
  SndFileDllLoaded:= (SndFileDLL <> 0);
end;

procedure UnLoadSndFileDll;
begin
  if (SndFileDLL <> 0) then
    begin
      FreeLibrary( SndFileDll);
      SndFileDll:= 0;
    end;
end;

function LoadSndFileDll:boolean;
begin
  if (SndFileDLL = 0) then
    begin
      SndFileDLL:= LoadLibrary( LibName);

      if (SndFileDLL <> 0) then
        begin
          // ol' Dephi 5 style was ..
          //@lsf_open            := GetProcAddress( SndFileDll, 'sf_open');
          //@lsf_open_fd         := ...
          lsf_open_native     := Tlsf_open_native(   GetProcAddress( SndFileDll, 'sf_open'));
          lsf_open_fd         := Tlsf_open_fd(       GetProcAddress( SndFileDll, 'sf_open_fd'));
          lsf_error           := Tlsf_error(         GetProcAddress( SndFileDll, 'sf_error'));
          lsf_strerror        := Tlsf_strerror(      GetProcAddress( SndFileDll, 'sf_strerror'));
          lsf_error_number    := Tlsf_error_number(  GetProcAddress( SndFileDll, 'sf_error_number'));
          lsf_perror          := Tlsf_perror(        GetProcAddress( SndFileDll, 'sf_perror'));
          lsf_error_str       := Tlsf_error_str(     GetProcAddress( SndFileDll, 'sf_error_str'));
          lsf_command         := Tlsf_command(       GetProcAddress( SndFileDll, 'sf_command'));
          lsf_format_check    := Tlsf_format_check(  GetProcAddress( SndFileDll, 'sf_format_check'));
          lsf_seek            := Tlsf_seek(          GetProcAddress( SndFileDll, 'sf_seek'));
          lsf_set_string      := Tlsf_set_string(    GetProcAddress( SndFileDll, 'sf_set_string'));
          lsf_get_string      := Tlsf_get_string(    GetProcAddress( SndFileDll, 'sf_get_string'));
          lsf_read_raw        := Tlsf_read_raw(      GetProcAddress( SndFileDll, 'sf_read_raw'));
          lsf_write_raw       := Tlsf_write_raw(     GetProcAddress( SndFileDll, 'sf_write_raw'));
          lsf_readf_short     := Tlsf_readf_short(   GetProcAddress( SndFileDll, 'sf_readf_short'));
          lsf_writef_short    := Tlsf_writef_short(  GetProcAddress( SndFileDll, 'sf_writef_short'));
          lsf_readf_int       := Tlsf_readf_int(     GetProcAddress( SndFileDll, 'sf_readf_int'));
          lsf_writef_int      := Tlsf_writef_int(    GetProcAddress( SndFileDll, 'sf_writef_int'));
          lsf_readf_float     := Tlsf_readf_float(   GetProcAddress( SndFileDll, 'sf_readf_float'));
          lsf_writef_float    := Tlsf_writef_float(  GetProcAddress( SndFileDll, 'sf_writef_float'));
          lsf_readf_double    := Tlsf_readf_double(  GetProcAddress( SndFileDll, 'sf_readf_double'));
          lsf_writef_double   := Tlsf_writef_double( GetProcAddress( SndFileDll, 'sf_writef_double'));
          lsf_read_short      := Tlsf_read_short(    GetProcAddress( SndFileDll, 'sf_read_short'));
          lsf_write_short     := Tlsf_write_short(   GetProcAddress( SndFileDll, 'sf_write_short'));
          lsf_read_int        := Tlsf_read_int(      GetProcAddress( SndFileDll, 'sf_read_int'));
          lsf_write_int       := Tlsf_write_int(     GetProcAddress( SndFileDll, 'sf_write_int'));
          lsf_read_float      := Tlsf_read_float(    GetProcAddress( SndFileDll, 'sf_read_float'));
          lsf_write_float     := Tlsf_write_float(   GetProcAddress( SndFileDll, 'sf_write_float'));
          lsf_read_double     := Tlsf_read_double(   GetProcAddress( SndFileDll, 'sf_read_double'));
          lsf_write_double    := Tlsf_write_double(  GetProcAddress( SndFileDll, 'sf_write_double'));
          lsf_close           := Tlsf_close(         GetProcAddress( SndFileDll, 'sf_close'));
          Result:= TRUE;
        end
      else Result:= FALSE;
   end;
end;


end.
